// $Id: SSH.cc 3660 2006-10-02 19:05:43Z vern $
//
// Copyright (c) 2001, 2002
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#include "config.h"

#include <ctype.h>

#include "NetVar.h"
#include "SSH.h"
#include "Event.h"
#include "ContentLine.h"

SSH_Analyzer::SSH_Analyzer(Connection* c)
: TCP_ApplicationAnalyzer(AnalyzerTag::SSH, c)
	{
	orig = new ContentLine_Analyzer(c, true);
	orig->SetSkipPartial(true);
	orig->SetCRLFAsEOL(LF_as_EOL);
	AddSupportAnalyzer(orig);

	resp = new ContentLine_Analyzer(c, false);
	resp->SetSkipPartial(true);
	resp->SetCRLFAsEOL(LF_as_EOL);
	AddSupportAnalyzer(resp);
	}

void SSH_Analyzer::DeliverStream(int length, const u_char* data, bool is_orig)
	{
	TCP_ApplicationAnalyzer::DeliverStream(length, data, is_orig);

	// We're all done processing this endpoint - flag it as such,
	// before we even determine whether we have any event generation
	// work to do, to make sure we don't do any further work on it.
	if ( is_orig )
		orig->SetSkipDeliveries(true);
	else
		resp->SetSkipDeliveries(true);

	if ( TCP() )
		{
		// Don't try to parse version if there has already been a gap.
		TCP_Endpoint* endp = is_orig ? TCP()->Orig() : TCP()->Resp();
		if ( endp->HadGap() )
			return;
		}

	const char* line = (const char*) data;

	// The SSH identification looks like this:
	//
	//     SSH-<protocolmajor>.<protocolminor>-<version>\n
	//
	// We're interested in the "version" part here.
	
	if ( length < 4 || memcmp(line, "SSH-", 4) != 0 )
		{
		Weird("malformed_ssh_identification");
		ProtocolViolation("malformed ssh identification");
		return;
		}
	
	int i;
	for ( i = 4; i < length && line[i] != '-'; ++i )
		;
	
	if ( TCP() )
		{
		if ( length >= i )
			{
			const uint32* dst;
			if ( is_orig )
				dst = TCP()->Orig()->dst_addr;
			else
				dst = TCP()->Resp()->dst_addr;

			if ( Conn()->VersionFoundEvent(dst, line + i,
							length - i) )
				ProtocolConfirmation();
			else
				ProtocolViolation("malformed ssh version");
			}
		else
			{
			Weird("malformed_ssh_version");
			ProtocolViolation("malformed ssh version");
			}
		}

	// Generate SSH events.
	EventHandlerPtr event = is_orig ?
				ssh_client_version : ssh_server_version;
	if ( ! event )
		return;

	val_list* vl = new val_list;
	vl->append(BuildConnVal());
	vl->append(new StringVal(length, line));

	ConnectionEvent(event, vl);
	}
